#!/bin/bash

# Extracts MB stats from the top N rows of an image file
# Outputs tables in text format ready for tab_to_bash.sh
# Usage: extract_stats.sh imagefile num_rows

# BUG: gaps need to be filled and outliers removed manually

imagefile="$1"
numrows="$2"

# Run ffmpeg and generate debug output
../FFmpeg-spacexdebug1/ffmpeg -threads 1 -debug mb_pos_size -err_detect ignore_err -s:0 704:480 -i $imagefile -f image2 img-%03d.png 2>debug.txt

lastrow=$((numrows - 1))
mbcount=$((numrows * 44))

# MB lengths
rm -f table_mb_length.data
for row in `seq 0 $lastrow ` ; do
	row0=`printf %02d $row`
	grep "pos/size: 0 000:[^:]*:$row0:" debug.txt | cut -d ' ' -f 8 >>table_mb_length.data
done
sed 's^$^ / 10^' table_mb_length.data | bc | sort -g | uniq -c | sed -e 's/^ *//' | sed -e "s/\$/ $mbcount/" >table_mb_length.tab

exit

# get cbpc
rm -f table_cbpc.data
for row in `seq 0 $lastrow` ; do
	row0=`printf %02d $row`
	grep "BITLOG:[^:]*:$row0:cbpc" debug.txt | cut -d : -f 7 >>table_cbpc.data
done
cat table_cbpc.data | sort -g | uniq -c | sed -e 's/^ *//' | sed -e "s/\$/ $mbcount/" >table_cbpc.tab


# get cbpy
rm -f table_cbpy.data
for row in `seq 0 $lastrow` ; do
	row0=`printf %02d $row`
	grep "BITLOG:[^:]*:$row0:cbpc" debug.txt | cut -d : -f 7 >>table_cbpy.data
done
cat table_cbpy.data | sort -g | uniq -c | sed -e 's/^ *//' | sed -e "s/\$/ $mbcount/" >table_cbpy.tab


# dc_lum
dclumcount=$((mbcount * 4))
rm -f table_dc_lum.data
for row in `seq 0 $lastrow` ; do
	row0=`printf %02d $row`
	grep "BITLOG:[^:]*:$row0:dc_lum:" debug.txt | cut -d : -f 8 >>table_dc_lum.data
done
cat table_dc_lum.data | sort -g | uniq -c | sed -e 's/^ *//' | sed -e "s/\$/ $dclumcount/" >table_dc_lum.tab


# dc_lum_level
dclumlevelcount=$((mbcount * 6))
rm -f table_dc_lum_level.data
for row in `seq 0 $lastrow` ; do
	row0=`printf %02d $row`
	grep "BITLOG:[^:]*:$row0:dc_lum_level:" debug.txt | cut -d : -f 8 >>table_dc_lum_level.data
done
cat table_dc_lum_level.data | sort -g | uniq -c | sed -e 's/^ *//' | sed -e "s/\$/ $dclumlevelcount/" >table_dc_lum_level.tab

# dc_chrom
dcchromcount=$((mbcount * 2))
rm -f table_dc_chrom.data
for row in `seq 0 $lastrow` ; do
	row0=`printf %02d $row`
	grep "BITLOG:[^:]*:$row0:dc_chrom:" debug.txt | cut -d : -f 8 >>table_dc_chrom.data
done
cat table_dc_chrom.data | sort -g | uniq -c | sed -e 's/^ *//' | sed -e "s/\$/ $dcchromcount/" >table_dc_chrom.tab

# dct_run
rm -f table_dct_run.data
for row in `seq 0 $lastrow` ; do
	row0=`printf %02d $row`
	grep "BITLOG:[^:]*:$row0:dct:" debug.txt | cut -d : -f 8 >>table_dct_run.data
done
dctruncount=`wc -l table_dct_run.data | cut -d ' ' -f 1`
cat table_dct_run.data | sort -g | uniq -c | sed -e 's/^ *//' | sed -e "s/\$/ $dctruncount/" >table_dct_run.tab

# dct_level
rm -f table_dct_level.data
for row in `seq 0 $lastrow` ; do
	row0=`printf %02d $row`
	grep "BITLOG:[^:]*:$row0:dct:" debug.txt | cut -d : -f 7 >>table_dct_level.data
done
dctlevelcount=`wc -l table_dct_level.data | cut -d ' ' -f 1`
cat table_dct_level.data | sort -g | uniq -c | sed -e 's/^ *//' | sed -e "s/\$/ $dctlevelcount/" >table_dct_level.tab


