#!/bin/bash

imagefile='iframe181.mpg4-img'

basemmb=','
#basemmb='X:553:80,X:668:80'
#basemmb='X:553:80,X:668:80,X:692:80,X:706:C0,X:732:80,X:746:C0,X:921:80,X:936:C0,X:939:80,X:953:C0,X:988:80,X:1002:C0'
#basemmb="$basemmb,X:993:80,X:1007:C0,X:1025:80,X:1021:80,X:1035:C0"

mb_x=0
mb_y=0

mb_x0=`printf %02d $mb_x`
mb_y0=`printf %02d $mb_y`

goodscore=-200000

### statistics tables ###
. ./tables.sh

function compute_score {
	# compute score based on debug output
	score=0

	# check that it decoded correctly
	if ! grep -q "MB pos/size: 0 000:$mb_x0:$mb_y0:" $1 ; then
		echo "Decoding error, score = -999999"
		score=-999999
	fi

	# get cbpc
	cbpc=`grep BITLOG:$mb_x0:$mb_y0:cbpc: $1 | cut -d ':' -f 7`
	cbpc=$((cbpc + table_cbpc_offset))
	if [ $cbpc -lt 0 -o $cbpc -gt $table_cbpc_max ] ; then
		score=$((score + table_cbpc_default))
	else
		score=$((score + table_cbpc[cbpc]))
	fi
	echo "cbpc: $cbpc, score: $score"

	# get cbpy
	cbpy=`grep BITLOG:$mb_x0:$mb_y0:cbpy: $1 | cut -d : -f 7`
	cbpy=$((cbpy + table_cbpy_offset))
	if [ $cbpy -lt 0 -o $cbpy -gt $table_cbpy_max ] ; then
		score=$((score + table_cbpy_default))
	else
		score=$((score + table_cbpy[cbpy]))
	fi
	echo "cbpy: $cbpy, score: $score"

	# get lum 0
	dclum0=`grep BITLOG:$mb_x0:$mb_y0:dc_lum:0 $1 | cut -d : -f 8`
	dclum0=$((dclum0 + table_dc_lum_offset))
	if [ $dclum0 -lt 0 -o $dclum0 -gt $table_dc_lum_max ] ; then
		score=$((score + table_dc_lum_default))
	else
		score=$((score + table_dc_lum[dclum0]))
	fi
	echo "dc_lum 0: $dclum0, score: $score"

	# get lum 1
	dclum1=`grep BITLOG:$mb_x0:$mb_y0:dc_lum:1 $1 | cut -d : -f 8`
	dclum1=$((dclum1 + table_dc_lum_offset))
	if [ $dclum1 -lt 0 -o $dclum1 -gt $table_dc_lum_max ] ; then
		score=$((score + table_dc_lum_default))
	else
		score=$((score + table_dc_lum[dclum1]))
	fi
	echo "dc_lum 1: $dclum1, score: $score"

	# get lum 2
	dclum2=`grep BITLOG:$mb_x0:$mb_y0:dc_lum:2 $1 | cut -d : -f 8`
	dclum2=$((dclum2 + table_dc_lum_offset))
	if [ $dclum2 -lt 0 -o $dclum2 -gt $table_dc_lum_max ] ; then
		score=$((score + table_dc_lum_default))
	else
		score=$((score + table_dc_lum[dclum2]))
	fi
	echo "dc_lum 2: $dclum2, score: $score"

	# get lum 3
	dclum3=`grep BITLOG:$mb_x0:$mb_y0:dc_lum:3 $1 | cut -d : -f 8`
	dclum3=$((dclum3 + table_dc_lum_offset))
	if [ $dclum3 -lt 0 -o $dclum3 -gt $table_dc_lum_max ] ; then
		score=$((score + table_dc_lum_default))
	else
		score=$((score + table_dc_lum[dclum3]))
	fi
	echo "dc_lum 3: $dclum3, score: $score"

	# get chrom 4
	dcchrom4=`grep BITLOG:$mb_x0:$mb_y0:dc_chrom:4 $1 | cut -d : -f 8`
	dcchrom4=$((dcchrom4 + table_dc_chrom_offset))
	if [ $dcchrom4 -lt 0 -o $dcchrom4 -gt $table_dc_chrom_max ] ; then
		score=$((score + table_dc_chrom_default))
	else
		score=$((score + table_dc_chrom[dcchrom4]))
	fi
	echo "dc_chrom 4: $dcchrom4, score: $score"

	# get chrom 5
	dcchrom5=`grep BITLOG:$mb_x0:$mb_y0:dc_chrom:4 $1 | cut -d : -f 8`
	dcchrom5=$((dcchrom5 + table_dc_chrom_offset))
	if [ $dcchrom5 -lt 0 -o $dcchrom5 -gt $table_dc_chrom_max ] ; then
		score=$((score + table_dc_chrom_default))
	else
		score=$((score + table_dc_chrom[dcchrom5]))
	fi
	echo "dc_chrom 5: $dcchrom5, score: $score"

	# get lum level 0
	dclumlev0=`grep BITLOG:$mb_x0:$mb_y0:dc_lum_level:0 $1 | cut -d : -f 8`
	dclumlev0=$((dclumlev0 + table_dc_lum_level_offset))
	if [ $dclumlev0 -lt 0 -o $dclumlev0 -gt $table_dc_lum_level_max ] ; then
		score=$((score + table_dc_lum_level_default))
	else
		score=$((score + table_dc_lum_level[dclumlev0]))
	fi
	echo "dc_lum_level 0: $dclumlev0, score: $score"

	# get lum level 1
	dclumlev1=`grep BITLOG:$mb_x0:$mb_y0:dc_lum_level:1 $1 | cut -d : -f 8`
	dclumlev1=$((dclumlev1 + table_dc_lum_level_offset))
	if [ $dclumlev1 -lt 0 -o $dclumlev1 -gt $table_dc_lum_level_max ] ; then
		score=$((score + table_dc_lum_level_default))
	else
		score=$((score + table_dc_lum_level[dclumlev1]))
	fi
	echo "dc_lum_level 1: $dclumlev1, score: $score"

	# get lum level 2
	dclumlev2=`grep BITLOG:$mb_x0:$mb_y0:dc_lum_level:2 $1 | cut -d : -f 8`
	dclumlev2=$((dclumlev2 + table_dc_lum_level_offset))
	if [ $dclumlev2 -lt 0 -o $dclumlev2 -gt $table_dc_lum_level_max ] ; then
		score=$((score + table_dc_lum_level_default))
	else
		score=$((score + table_dc_lum_level[dclumlev2]))
	fi
	echo "dc_lum_level 2: $dclumlev2, score: $score"

	# get lum level 3
	dclumlev3=`grep BITLOG:$mb_x0:$mb_y0:dc_lum_level:3 $1 | cut -d : -f 8`
	dclumlev3=$((dclumlev3 + table_dc_lum_level_offset))
	if [ $dclumlev3 -lt 0 -o $dclumlev3 -gt $table_dc_lum_level_max ] ; then
		score=$((score + table_dc_lum_level_default))
	else
		score=$((score + table_dc_lum_level[dclumlev3]))
	fi
	echo "dc_lum_level 3: $dclumlev3, score: $score"

	# get lum level 4
	dclumlev4=`grep BITLOG:$mb_x0:$mb_y0:dc_lum_level:4 $1 | cut -d : -f 8`
	dclumlev4=$((dclumlev4 + table_dc_lum_level_offset))
	if [ $dclumlev4 -lt 0 -o $dclumlev4 -gt $table_dc_lum_level_max ] ; then
		score=$((score + table_dc_lum_level_default))
	else
		score=$((score + table_dc_lum_level[dclumlev4]))
	fi
	echo "dc_lum_level 4: $dclumlev4, score: $score"

	# get lum level 5
	dclumlev5=`grep BITLOG:$mb_x0:$mb_y0:dc_lum_level:5 $1 | cut -d : -f 8`
	dclumlev5=$((dclumlev5 + table_dc_lum_level_offset))
	if [ $dclumlev5 -lt 0 -o $dclumlev5 -gt $table_dc_lum_level_max ] ; then
		score=$((score + table_dc_lum_level_default))
	else
		score=$((score + table_dc_lum_level[dclumlev5]))
	fi
	echo "dc_lum_level 5: $dclumlev5, score: $score"

	# get dct levels
	dctscore=0
	dctcount=`grep BITLOG:$mb_x0:$mb_y0:dct: $1 | wc -l`
	if [ $dctcount -gt 0 ] ; then
		for i in `seq 1 $dctcount` ; do
			# calculate score
			dctlev=`grep BITLOG:$mb_x0:$mb_y0:dct: $1 | head -n $i | tail -n 1 | cut -d : -f 7`
			dctlev=$((dctlev + table_dct_level_offset))
			if [ $dctlev -lt 0 -o $dctlev -gt $table_dct_level_max ] ; then
				dctscore=$((dctscore + table_dct_level_default))
			else
				dctscore=$((dctscore + table_dct_level[dctlev]))
			fi
		done
		# add dctscore / count to score
		score=$((score + 10 * dctscore / dctcount))
		echo "dct: $((10 * dctscore / dctcount)), score: $score"
	else
		echo "no dct"
		# TODO: check that cbpc and cbpy are both 0?
	fi

	# get top and left edge sum-of-squared-errors
	tledge=`../dev/mb_tl_edge_bl_edge_rmse $mb_x $mb_y <$2`
	score=$((score - 100 * tledge))
	echo "tledge: $((-100 * tledge)), score: $score"
}

rm output/*

while [ $mb_y -lt 30 ] ; do

	# do a base run to find next block and which bits it covers
	../FFmpeg-spacexdebug1/ffmpeg -threads 1 -mmb $basemmb -debug mb_pos_size -err_detect ignore_err -s:0 704:480 -i $imagefile -f image2 output/img-base-%03d.png 2>output/debug_base.txt

	score=0
	while [ $mb_y -lt 30 -a $score -gt $goodscore ] ; do
		echo -e "\ncol: $mb_x0, row: $mb_y0"
		compute_score "output/debug_base.txt" output/img-base-001.png
		refscore="$score"
		if [ $score -gt $goodscore ] ; then
			mb_x=$((mb_x + 1))
			mb_x0=`printf %02d $mb_x`
			if [ $mb_x -ge 44 ] ; then
				mb_y=$((mb_y + 1))
				mb_y0=`printf %02d $mb_y`
				mb_x=0
				mb_x0='00'
			fi
		fi
	done

	if [ $mb_y -lt 30 ] ; then
		firstbit=`grep "MB pos/size: [^ ]* 000:$mb_x0:$mb_y0:" output/debug_base.txt | sed -e "s%.* MB pos/size: [^ ]* 000:$mb_x0:$mb_y0:\([^ ]*\) .*%\1%"`
		mblen=`grep "MB pos/size: [^ ]* 000:$mb_x0:$mb_y0:" output/debug_base.txt | sed -e "s%.* MB pos/size: [^ ]* 000:$mb_x0:$mb_y0:\([^ ]*\) \([^ ]*\) .*%\2%"`
		lastbit=$((firstbit + mblen))

		echo "Base mmb now $basemmb, base score = $refscore"
		echo "Trying MB $mb_x0:$mb_y0 from $firstbit to $lastbit"

		if [ $lastbit == 0 ]; then exit ; fi

		rm -f output/scores_raw.txt
		for bit in `seq $firstbit $lastbit` ; do
			if ! echo "$mmb" | grep -q "X:$bit:80"  ; then
				secondbits=$((bit + 14))
				mmb="$basemmb,X:$bit:80,X:$secondbits:C0"

				echo -n "Trying mmb $mmb: score = "
				../FFmpeg-spacexdebug1/ffmpeg -threads 1 -mmb $mmb -debug mb_pos_size -err_detect ignore_err -s:0 704:480 -i $imagefile -f image2 output/img-$bit-%03d.png 2>output/debug_$bit.txt
#				score=`../dev/mb_jigsaw_count output/img-$bit-001.png`

				# compute score
				compute_score "output/debug_$bit.txt" output/img-$bit-001.png
				echo "$score"

				echo -e "$score\t$bit\timg-$bit-001.png" >>output/scores_raw.txt

				# remove output to conserve disk space
#				rm output/img-$bit-001.png
#				rm output/debug_$bit.txt
			fi
		done

		sort -g output/scores_raw.txt >output/scores.txt

		bestscore=`tail -n 1 output/scores.txt | cut -f 1`
		if [ $bestscore -gt $refscore ] ; then
			bestbit=`tail -n 1 output/scores.txt | cut -f 2`
			bestbit2=$((bestbit + 14))
			basemmb="$basemmb,X:$bestbit:80,X:$bestbit2:C0"
		else
			mb_x=$((mb_x + 1))
			mb_x0=`printf %02d $mb_x`
		fi
	fi
	exit
done

echo "Final mmb: $basemmb"

